; *******************************************************************
;   Plug in Libraries for the by 12 architecture                    *
;   Written and Tested using MPASM V1.40                            *
;********************************************************************
;   Filename:		PINCHANG.ASM                                *
;   Date:               05 November 1997                            *
;   File Version:       1.00                                        *
;********************************************************************
;  Macros:
; Wait on pin change
; Implements wait on pin change L -> H
;                               H -> L
;
;  Functions:
;                       OpenPinChangePort Defines the port and pin
;                                         to watch for pin change. 
;                                         with the rest of the TRIS mask.
;                       SWPWM12         - Advances PWM State machine
;
;  Internal Variables
;                       PWM_DutyCycle
;                       PWM_Sum
;                       PWM_Period
;********************************************************************
;   Revision History:                                               *
;      V1.00 - Beta release of Peripheral Libraries                 *
;********************************************************************
;   Notes:                                                          *
; *******************************************************************
;
;*****************************************************************
;	Function Name:	WaitOnPinChange                          *
;	Return Value:	none                                     *
;	Parameters:	Port and Pin                             *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	7 words                                  *
;	Description:	Monitors the specified port and pin and  *
;			waits until the pin changes before       *
;                       continuing.  Assumes the pin has been    *
;                       TRISd as an input prior to executing the *
;                       macro.                                   *
;*****************************************************************
WaitOnPinChange	macro   port, pin
        local  _HLTransition, _LHTransition, done

	btfss	port,pin	; what is the pin now?
	goto	LHTransition	; Low - watch for LH transition
HLTransition
	btfsc	port, pin	; High - wait for HL transition
	goto 	HLTransition
	goto	done
LHTransition
	btfss	port, pin	; High - wait for LH transition
	goto 	LHTransition
done
	endm

;*****************************************************************
;	Function Name:	WaitOnPinChangeLH                        *
;	Return Value:	none                                     *
;	Parameters:	Port and Pin                             *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	4 words                                  *
;	Description:	Monitors the specified port and pin and  *
;			waits until the pin changes from LOW to  *
;                       HIGH before continuing.  The pin must be *
;                       TRISd as an input prior to executing the *
;                       macro.                                   *
;*****************************************************************
WaitOnPinChangeLH	macro   port, pin
        local  loop1, loop2

loop1   btfsc	port,pin	; Wait for pin to go low first
	goto	loop1
loop2	btfss	port, pin	; now wait for pin to transition High
	goto 	loop2
	endm

;*****************************************************************
;	Function Name:	WaitOnPinChangeHL                        *
;	Return Value:	none                                     *
;	Parameters:	Port and Pin                             *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	4 words                                  *
;	Description:	Monitors the specified port and pin and  *
;			waits until the pin changes from HIGH to *
;                       LOW before continuing.  The pin must be  *
;                       TRISd as an input prior to executing the *
;                       macro.                                   *
;*****************************************************************
WaitOnPinChangeHL	macro   port, pin
        local  loop1, loop2
loop1	btfss	port,pin	; Wait for pin to transition high first
	goto	loop1
loop2	btfsc	port, pin	; Now wait for pin to transition low
	goto 	loop2
	endm
