; *******************************************************************
;   Plug in Libraries for the by 12 architecture                    *
;   Written and Tested using MPASM V1.40                            *
;********************************************************************
;   Filename:		SWPWM12.ASM                                 *
;   Date:               05 November 1997                            *
;   File Version:       1.00                                        *
;********************************************************************
;  Macros:
;                       SetPWMDutyCycle - Number of steps out of the
;                                         period that will be high
;                       SetPWMPeriod    - Set number of steps that
;                                         define the PWM cycle.
;
;  Functions:
;                       OpenSWPWM       - Defines the port and pin
;                                         that will be PWM'd, along
;                                         with the rest of the TRIS mask.
;                       SWPWM12         - Advances PWM State machine
;
;  Internal Variables
;                       PWM_DutyCycle
;                       PWM_Sum
;                       PWM_Period
;********************************************************************
;   Revision History:                                               *
;      V1.00 - Beta release of Peripheral Libraries                 *
;********************************************************************
;   Notes:                                                          *
; *******************************************************************
; Change these definitions as needed and reassemble the library     *
;                                                                   *
        title "Software PWM functions for by12 architecture"
        list p=pic16c5x
#include <p16c5x.inc>
;

#define PWM_PORT    PORTA
#define PWM_PIN     3
#define TRISMASK    0xff


	if PWM_PORT < PORTA || PWM_PORT > 7
		messg "Invalid port value"
	endif
	if PWM_PIN > 7
		messg "Invalid pin value"
	endif

portmask = ~(0x01 << PWM_PIN) & TRISMASK

	udata
PWM_DutyCycle	res	1
PWM_Sum		res	1
PWM_Period	res	1
        global  PWM_DutyCycle
        global  PWM_Period

        code
;
;
;*****************************************************************
;	Function Name:	OpenSWPWM                                *
;	Return Value:	none                                     *
;	Parameters:	None (parameters set at assembly time)   *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	7 words                                  *
;	Description:	Sets the TRIS register for the PWMPORT   *
;			such that the PWM_PIN is an output, and  *
;                       the rest of the bits are as specified by *
;                       TRISMASK.                                *
;*****************************************************************
OpenSWPWM
        global  OpenSWPWM
	bcf	PWM_PORT,PWM_PIN	; Start it off low
	movlw	portmask
	tris	PWM_PORT		; Set PWM pin for output
	clrf	PWM_Sum		; clear the sum
        clrf    PWM_Period      ; max period (256)
        clrf    PWM_DutyCycle   ; min duty cycle (0)
	retlw   0

; Included for sake of completeness
;CloseSWPWM
;       global  CloseSWPWM
;       bcf	PWM_PORT,PWM_PIN	; Set output low
;       clrf    PWM_DutyCycle   ; min duty cycle (0)
;       retlw   0
;
;*****************************************************************
;	Function Name:	SWPWM12                                  *
;	Return Value:	none                                     *
;	Parameters:	None (parameters set at assembly time)   *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	5 words                                  *
;	Description:	Advances the state of the PWM state      *
;			machine and changes the state of the pin *
;                       if necessary.                            *
;*****************************************************************
SWPWM12
        global  SWPWM12
	incf	PWM_Sum,f 		; bump the counter

        movf    PWM_Period,w    ; compare the sum against the period.
        subwf   PWM_Sum,w
        btfsc   STATUS,Z        ; if the sum == period,
        clrf    PWM_Sum         ; reset the sum.

	movf	PWM_DutyCycle,w
	subwf	PWM_Sum,w	; compare against the duty cycle
	btfsc	STATUS,C	; If Sum < duty cycle
	goto	pwmlow
pwmhi	bsf	PWM_PORT,PWM_PIN
	retlw   0

pwmlow  bcf	PWM_PORT,PWM_PIN
	retlw	0

        end
