; *******************************************************************
;   Plug in Libraries for the by 12 architecture                    *
;   Written and Tested using MPASM V1.40                            *
;********************************************************************
;   Filename:		SWSPI12.ASM                                 *
;   Date:               05 November 1997                            *
;   File Version:       1.00                                        *
;********************************************************************
;  Macros:
;                       SetCSSWSPI      - Number of steps out of the
;                                         period that will be high
;                       ClearCSSWSPI    - Set number of steps that
;                                         define the PWM cycle.
;
;  Functions:
;                       OpenSWSPI       - Defines the port and pins
;                                         to use for SPI communication.
;                                         with the rest of the TRIS mask.
;                       putcSWSPI       - Writes and reads a byte to/from
;                                         the SPI device.
;********************************************************************
;   Revision History:                                               *
;      V1.00 - Beta release of Peripheral Libraries                 *
;********************************************************************
;   Notes:                                                          *
;
; To use the software spi routines, the user must define the port and
; tris register for each of the CS, DIN, DOUT, and SCK pins.  The SPI
; mode must also be defined (MODE0, MODE1, MODE2, MODE3).
;
; Define the port and pin for each of the software SPI pins
; The chip select pin CS must have a port and tris definition.
; The data in pin DIN must have a port and tris definition.
; The data out pin DOUT must have a port and tris definition.
; The clock pin SCK must have a port and tris definition.

#define SW_SPI_PORT       PORTB
#define SW_CS_PIN     0		; Chip Select
#define SW_DIN_PIN    2		; Data in
#define SW_DOUT_PIN   1		; Data out
#define SW_SCK_PIN    3		; Clock
#define SW_SPI_TRISMASK   0xff  ; remaining pins inputs.

; Define the mode for software SPI:
; Refer to the SPI module for PIC17C756 for definitions of CKP and CKE
; Only one mode can be uncommented, otherwise the software will not work
; CKP:  Clock Polarity Select  1 - Idle state high, 0 - idle state low
; CKE:  Clock Edge Select.
; If CKP = 1; CKE - 1 Data tx'd on falling edge,  0 - Data Tx'd on rising edge
; If CKP = 0; CKE - 1 Data tx'd on rising edge,  0 - Data Tx'd on falling edge
#define MODE0	; CKP=0,CKE=0: Idle Low,  data tx'd on falling edge
;#define MODE1	; CKP=1,CKE=0: Idle High, data tx'd on rising edge
;#define MODE2	; CKP=0,CKE=1: Idle Low,  data tx'd on rising edge
;#define MODE3	; CKP=1,CKE=1: Idle High, Data tx'd on falling edge
;
; **********************************************************************
	list p=pic16c5x
#include <p16c5x.inc>

	if SW_SPI_PORT < 5 || SW_SPI_PORT > 7
		messg "Invalid port value"
	endif

	if (SW_DIN_PIN > 7) || ((SW_SPI_PORT == 5) && (SW_DIN_PIN > 4))
		messg "Invalid DataIn pin value"
	endif

	if (SW_DOUT_PIN > 7) || ((SW_SPI_PORT == 5) && (SW_DOUT_PIN > 4))
		messg "Invalid DataOut pin value"
	endif

	if (SW_SCK_PIN > 7) || ((SW_SPI_PORT == 5) && (SW_SCK_PIN > 4))
		messg "Invalid Clock pin value"
	endif

	if (SW_CS_PIN > 7) || ((SW_SPI_PORT == 5) && (SW_CS_PIN > 4))
		messg "Invalid Chip Select pin value"
	endif

dimask = 0x01 << SW_DIN_PIN
domask = 0x01 << SW_DOUT_PIN
clkmask = 0x01 << SW_SCK_PIN
cselmask = 0x01 << SW_CS_PIN
_SPI_TRIS_MASK equ (SW_SPI_TRISMASK | dimask) & (~domask) & (~clkmask) & (~cselmask)

        udata
dataout  res 1
        global  dataout

        udata_ovr
BitCount res 1

;*****************************************************************
;	Function Name:	OpenSWSPI                                    *
; 	Return Value:	None                                         *
;	Parameters:	Pins must be defined at assembly time        *
;	RAM Usage:	0 bytes                                      *
;	ROM Usage:	~10 words                                    *
;	Description:	This routine configures the I/O pins for SPI *
;					operation.                                   *
;****************************************************************/

        code
OpenSWSPI
        global  OpenSWSPI
        MOVLW   _SPI_TRIS_MASK
	tris    SW_SPI_PORT

	bsf     SW_SPI_PORT, SW_CS_PIN	; Make the CS pin high
	bsf     SW_SPI_PORT, SW_DIN_PIN ; Make the DIN pin high

        ifdef MODE0			;	// Mode 0
	bcf     SW_SPI_PORT, SW_DOUT_PIN        ; Make the Dout and SCK pin low
	bcf     SW_SPI_PORT, SW_SCK_PIN
        endif

        ifdef MODE1			;	// Mode 1
	bcf     SW_SPI_PORT, SW_DOUT_PIN        ; Make the Dout and SCK pin low
	bsf     SW_SPI_PORT, SW_SCK_PIN
        endif

        ifdef MODE2			;	// Mode 2
	bcf     SW_SPI_PORT, SW_DOUT_PIN        ; Make the Dout and SCK pin low
	bcf     SW_SPI_PORT, SW_SCK_PIN
        endif

        ifdef MODE3			  ; Mode 3
	bcf     SW_SPI_PORT, SW_DOUT_PIN  ; Make the Dout and SCK pin low
	bsf     SW_SPI_PORT, SW_SCK_PIN
        endif

        MOVLW   _SPI_TRIS_MASK
	tris    SW_SPI_PORT
;       TRIS_SW_CS_PIN = 0;			// Make the CS pin an output
;	TRIS_SW_DIN_PIN = 1;		// Make the DIN pin an input
;	TRIS_SW_DOUT_PIN = 0;		// Make the DOUT pin an output
;	TRIS_SW_SCK_PIN = 0;		// Make the SCK pin an output

	retlw 0;


;*****************************************************************
;	Function Name:	putcSWSPI                                *
;	Return Value:	dataout: received data                   *
;	Parameters:	dataout: data to transmit                *
;	RAM Usage:		2 bytes                          *
;	ROM Usage:		~43 words                        *
;	Description:	Writes and reads a byte simultaneously   *
;*****************************************************************
putcSWSPI
        global  putcSWSPI
        movwf   dataout

        movlw   0x08
        movwf   BitCount;	BitCount = 8;	// Do 8-bits

#ifdef MODE0				; Mode 0
; SCK idles low
; Data output after falling edge of SCK
; Data sampled before rising edge of SCK
        bcf     SW_SPI_PORT, SW_DOUT_PIN        ; SW_DOUT_PIN = 0; //Set Dout to MSB of data

	movf    dataout,w       ;
        andlw   0x80            ; if(data&0x80)
        btfss   STATUS,Z
        bsf     SW_SPI_PORT, SW_DOUT_PIN        ; SW_DOUT_PIN = 1; //Set Dout to MSB of data
	nop;						// Adjust for jitter
	nop
RWSPILoop
        bcf     STATUS,C        ;Set the carry bit according
        btfsc   SW_SPI_PORT, SW_DIN_PIN
        bsf     STATUS,C
        bsf     SW_SPI_PORT, SW_SCK_PIN
	rlf    dataout,f       ; Rotate the carry into the data byte
	nop			;		// Produces a 50% duty cycle clock
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
        bcf     SW_SPI_PORT, SW_SCK_PIN ; Clear the SCK pin
         bcf     SW_SPI_PORT, SW_DOUT_PIN; Set Dout to the next bit according to
	movf    dataout,w       ;
        andlw   0x80            ; if(data&0x80)
        btfss   STATUS,Z
        bsf     SW_SPI_PORT, SW_DOUT_PIN
        decfsz  BitCount,f      ; Count iterations through loop
        goto    RWSPILoop           ; while(BitCount);
#endif

#ifdef MODE1       ; Mode1
; SCK idles high
; Data output after rising edge of SCK
; Data sampled before falling edge of SCK
        bcf     SW_SPI_PORT, SW_DOUT_PIN        ; SW_DOUT_PIN = 0; //Set Dout to MSB of data

	movf    dataout,w       ;
        andlw   0x80            ; if(data&0x80)
        btfss   STATUS,Z
        bsf     SW_SPI_PORT, SW_DOUT_PIN        ; SW_DOUT_PIN = 1; //Set Dout to MSB of data
	nop;						// Adjust for jitter
	nop
RWSPILoop
        bcf     STATUS,C        ;Set the carry bit according
        btfsc   SW_SPI_PORT, SW_DIN_PIN
        bsf     STATUS,C
        bcf     SW_SPI_PORT, SW_SCK_PIN
	rlf    dataout,f       ; Rotate the carry into the data byte
	nop			; Produces a 50% duty cycle clock
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
        bsf     SW_SPI_PORT, SW_SCK_PIN ; Clear the SCK pin
        bcf     SW_SPI_PORT, SW_DOUT_PIN; Set Dout to the next bit according to
	movf    dataout,w       ;
        andlw   0x80            ; if(data&0x80)
        btfss   STATUS,Z
        bsf     SW_SPI_PORT, SW_DOUT_PIN
        decfsz  BitCount,f        ; Count iterations through loop
        goto    RWSPILoop           ; while(BitCount);
#endif

#ifDEF MODE2                    ; Mode2
; SCK idles low
; Data output after rising edge of SCK
; Data sampled after falling edge of SCK
RWSPILoop
        bsf     SW_SPI_PORT, SW_SCK_PIN ; Set the SCK pin
        bcf     SW_SPI_PORT, SW_DOUT_PIN ; Set the Dout pin according to
	movf    dataout,w       ;
        andlw   0x80            ; if(data&0x80)
        btfss   STATUS,Z
        bsf     SW_SPI_PORT, SW_DOUT_PIN
        bcf     STATUS,C        ;Set the carry bit according

        btfsc   SW_SPI_PORT, SW_DIN_PIN
        bsf     STATUS,C
        bcf     SW_SPI_PORT, SW_SCK_PIN ; Set the SCK pin
	rlf     dataout,f       ; Rotate the carry into the data byte
	nop			; Produces a 50% duty cycle clock
	nop
	nop
	nop
	nop
        decfsz  BitCount,f      ; Count iterations through loop
        goto    RWSPILoop           ; while(BitCount);
#endif

#ifdef MODE3                    ; Mode 3
; SCK idles high
; Data output after falling edge of SCK
; Data sampled before rising edge of SCK   ; Clear the SCK pin
RWSPILoop
        bcf     SW_SPI_PORT, SW_SCK_PIN
        bcf     SW_SPI_PORT, SW_DOUT_PIN   ; Set the Dout pin according to 

	movf    dataout,w       ;
        andlw   0x80            ; if(data&0x80)
        btfss   STATUS,Z
        bsf     SW_SPI_PORT, SW_DOUT_PIN
        bcf     STATUS,C        ;Set the carry bit according

        btfsc   SW_SPI_PORT, SW_DIN_PIN
        bsf     STATUS,C
        bsf     SW_SPI_PORT, SW_SCK_PIN ; Set the SCK pin
	rlf     dataout,f       ; Rotate the carry into the data byte
	nop			; Produces a 50% duty cycle clock
	nop
	nop
	nop
	nop
        decfsz  BitCount,f        ; Count iterations through loop
        goto    RWSPILoop           ; while(BitCount);
#endif

	retlw 0                 ; Return the received data
        end

